import ICommand from '../Interfaces/ICommand';
import { Message, Collection } from 'discord.js';
import IContext from '../Interfaces/IContext';
/**
 * @module Command
 * @preferred
 */
declare abstract class Command implements ICommand {
    AllowedGuilds: string[];
    AllowedChannels: string[];
    AllowedRoles: string[];
    AllowedUsers: string[];
    Data: any;
    RequiresDatabase: boolean;
    Signature: string;
    BotContext: IContext;
    private Modified;
    CooldownRate: number;
    LastRun: number;
    constructor(channels: string[], roles: string[], users: string[], dbRequired?: boolean);
    LocalData: any;
    readonly HasLocalData: boolean;
    abstract Run(message: Message): Promise<any>;
    Name(): string;
    Namespace(): string;
    private HasLocalField;
    protected ValidateChannel(channelId: string): boolean;
    protected ValidateRoles(roles: Collection<any, any>): boolean;
    protected ValidateUsers(user: string): boolean;
    private GetValidationPermission;
    private ModifyPermissions;
    AddAllowedChannel(channelId: string, local?: boolean): void;
    AddAllowedRole(roleId: string, local?: boolean): void;
    AddAllowedUser(user: string, local?: boolean): void;
    Call(message: Message, isOwner: boolean): Promise<any>;
    /**
     * Returns false if command is on cooldown
     */
    CheckCooldown(): boolean;
    ContextInjection(context?: any): {};
    private GetParameterizedContext;
    private GetGenericContext;
    GetContext(message: Message, parameterized?: boolean): any;
    RemoveAllowedChannel(channelId: string, local?: boolean): void;
    RemoveAllowedRole(roleId: string, local?: boolean): void;
    RemoveAllowedUsers(user: string, local?: boolean): void;
    Save(): Promise<any>;
    Validate(message: Message): boolean;
}
export default Command;
